@@ -3,9 +3,10 @@ package Async::Event::Interval;
 use warnings;
 use strict;
 
-our $VERSION = '1.02';
+our $VERSION = '1.03';
 
 use Carp qw(croak);
+use IPC::Shareable;
 use Parallel::ForkManager;
 
 $SIG{CHLD} = "IGNORE";
@@ -14,31 +15,85 @@ $SIG{__WARN__} = sub {
     warn $warn if $warn !~ /^child process/;
 };
 
+my $id = 0;
+my %events;
+
+*restart = \&start;
+
 sub new {
     my $self = bless {}, shift;
-    $self->{pm} = Parallel::ForkManager->new(1);
-    $self->_set(@_);
-    $self->{started} = 0;
+    $self->_pm;
+    $self->_setup(@_);
+    $self->_started(0);
+
+    $self->id($id);
+    $id++;
+    $events{$self->id} = {};
+
     return $self;
 }
+sub events {
+    return \%events;
+}
+sub id {
+    my ($self, $id) = @_;
+    $self->{id} = $id if defined $id;
+    return $_[0]->{id};
+}
+sub info {
+    my ($self) = @_;
+    return $self->events()->{$self->id};
+}
+sub shared_scalar {
+    my ($self) = @_;
+
+    my $shm_key = _rand_shm_key();
+
+    tie my $scalar, 'IPC::Shareable', $shm_key, {create => 1, destroy => 1};
+
+    $events{$self->id}->{shared_scalars}{$shm_key} = \$scalar;
+
+    return \$scalar;
+}
 sub start {
     my $self = shift;
-    if ($self->{started}){
-        warn "event already running...\n";
+    if ($self->_started){
+        warn "Event already running...\n";
         return;
     }
-    $self->{started} = 1;
+    $self->_started(1);
     $self->_event;
 }
-*restart = \&start;
+sub status {
+    my $self = shift;
+
+    if ($self->_started){
+        if (! $self->_pid){
+            croak "Event is started, but no PID can be found. This is a " .
+                "fatal error. Exiting...\n";
+        }
+        if ($self->_pid > 0){
+            if (kill 0, $self->_pid){
+                return $self->_pid;
+            }
+            else {
+                # proc must have crashed
+                $self->_started(0);
+                $self->_pid(-99);
+                return -1;
+            }
+        }
+    }
+    return -1 if defined $self->_pid && $self->_pid == -99;
+    return 0;
+}
 sub stop {
     my $self = shift;
 
     if ($self->_pid){
         kill 9, $self->_pid;
 
-        $self->{started} = 0;
-        $self->{stop} = 1;
+        $self->_started(0);
 
         # time to ensure the proc was killed
 
@@ -50,39 +105,36 @@ sub stop {
         }
     }
 }
-sub status {
-    my $self = shift;
-
-    if ($self->{started}){
-        if (! $self->_pid){
-            croak "Event is started, but no PID can be found. This is a " .
-                  "fatal error. Exiting...\n";
-        }
-        if ($self->_pid > 0){
-            if (kill 0, $self->_pid){
-                return $self->_pid;
-            }
-            else {
-                # proc must have crashed
-                $self->{started} = 0;
-                $self->_pid(-99);
-                return -1;
-            }
-        }
-    }
-    return -1 if defined $self->_pid && $self->_pid == -99;
-    return 0;
-}
 sub waiting {
     my ($self) = @_;
     return 1 if ! $self->status || $self->status == -1;
     return 0;
 }
+
+sub _args {
+    my ($self, $args) = @_;
+
+    if (defined $args) {
+        $self->{args} = $args;
+    }
+
+    return $self->{args};
+}
+sub _cb {
+    my ($self, $cb) = @_;
+
+    if (defined $cb) {
+        croak "Callback must be a code reference." if ref $cb ne 'CODE';
+        $self->{cb} = $cb;
+    }
+
+    return $self->{cb};
+}
 sub _event {
     my $self = shift;
     
     for (0..1){
-        my $pid = $self->{pm}->start;
+        my $pid = $self->_pm->start;
         if ($pid){
             # this is the parent process
             $self->_pid($pid);
@@ -95,42 +147,75 @@ sub _event {
 
         # if no interval, run only once
 
-        if ($self->{interval}) {
+        if ($self->_interval) {
             while (1) {
-                $self->{cb}->(@{$self->{args}});
-                sleep $self->{interval};
+                $self->_cb->(@{$self->_args});
+                select(undef, undef, undef, $self->_interval);
             }
         }
         else {
-            $self->{cb}->(@{$self->{args}});
+            $self->_cb->(@{$self->_args});
         }
 
-        $self->{pm}->finish;
+        $self->_pm->finish;
     }
 }
+sub _interval {
+    my ($self, $interval) = @_;
+
+    if (defined $interval) {
+        $self->{interval} = $interval;
+    }
+
+    return $self->{interval};
+}
+sub _pm {
+    my ($self) = @_;
+
+    if (! exists $self->{pm}) {
+        $self->{pm} = Parallel::ForkManager->new(1);
+    }
+
+    return $self->{pm};
+}
 sub _pid {
     my ($self, $pid) = @_;
     $self->{pid} = $pid if defined $pid;
+    $events{$self->id}->{pid} = $self->{pid};
     return $self->{pid} || undef;
 }
-sub _set {
-    my ($self, $interval, $cb, @args) = @_;
+sub _rand_shm_key {
+    my $key_str;
 
-    $self->{interval} = $interval;
-    $self->{cb} = $cb;
-    $self->{args} = \@args;
+    for (0..3) {
+        $key_str .= ('A'..'Z')[rand(26)];
+    }
+
+   return $key_str;
+}
+sub _setup {
+    my ($self, $interval, $cb, @args) = @_;
+    $self->_interval($interval);
+    $self->_cb($cb);
+    $self->_args(\@args);
+}
+sub _started {
+    my ($self, $started) = @_;
+    $self->{started} = $started if defined $started;
+    return $self->{started};
 }
 sub DESTROY {
     $_[0]->stop if $_[0]->_pid;
 }
 sub _vim{}
+
 1;
 
 __END__
 
 =head1 NAME
 
-Async::Event::Interval - Extremely simple timed asynchronous events
+Async::Event::Interval - Timed and one-off asynchronous events
 
 =for html
 <a href="https://github.com/stevieb9/async-event-interval/actions"><img src="https://github.com/stevieb9/async-event-interval/workflows/CI/badge.svg"/></a>
@@ -139,57 +224,42 @@ Async::Event::Interval - Extremely simple timed asynchronous events
 
 =head1 SYNOPSIS
 
-A simple event. Multiple events can be simultaneously used. For an example using
-an event that can share data with the main application, examples of how to
-handle event crashes, and how to send parameters to your event callback, see 
-L</EXAMPLES>.
+A simple event that updates JSON data from a website using a shared scalar
+variable, while allowing the main application to continue running in the
+foreground. Multiple events can be simultaneously used if desired.
+
+See L</EXAMPLES> for other various functionality of this module.
+
+    use warnings;
+    use strict;
 
     use Async::Event::Interval;
 
-    my $event = Async::Event::Interval->new(
-        1.5, 
-        \&callback
-    );
+    my $event = Async::Event::Interval->new(2, \&callback);
+
+    my $shared_scalar_json = $event->shared_scalar;
 
     $event->start;
 
-    for (1..10){
-        print "$_: in main loop\n";
+    while (1) {
+        print "$$shared_scalar_json\n" if defined $$shared_scalar_json;
 
-        $event->stop if $_ == 3;
-        $event->start if $_ == 7;
-
-        if ($event->status){
-            print "event is running\n";
-        }
-
-        if ($event->status == -1){
-            print "event has crashed... restarting it\n";
-            $event->restart;
-        }
-
-        sleep 1;
+        # Do other things
     }
 
     sub callback {
-        print "timed event callback\n";
+        $$shared_scalar_json = ...; # Fetch JSON from website
     }
 
 =head1 DESCRIPTION
 
-Very basic implementation of asynchronous events that are triggered by a timed
-interval. If no time is specified, we'll run the event only once.
-
-Variables are not shared between the main application and the event. To do that,
-you'll need to use some form of memory sharing, such as L<IPC::Shareable>. See
-L</EXAMPLES> for an example. At this time, there is no real parameter passing or
-ability to return values. As I said... basic.
-
-Each event is simply a separate forked process, which runs in a while loop.
+Very basic implementation of asynchronous events with shared variables that are
+triggered by a timed interval. If no time is specified, we'll run the event only
+once.
 
 =head1 METHODS
 
-=head2 new($delay, $callback)
+=head2 new($delay, $callback, @params)
 
 Returns a new C<Async::Event::Interval> object. Does not create the event. Use
 C<start> for that.
@@ -207,6 +277,14 @@ we'll simply run the event once and stop.
 Mandatory: A reference to a subroutine that will be called every time the
 interval expires.
 
+    @params
+
+Optional, List: A list of parameters to pass to the callback. Note that these
+are not shared parameters and are a copy only, so changes to them in the main
+code will not be seen in the event, and vice-versa. See L</shared_scalar> if
+you'd like to use variables that can be shared between the main application and
+the events.
+
 =head2 start
 
 Starts the event timer. Each time the interval is reached, the event callback
@@ -230,12 +308,55 @@ isn't, and C<-1> if the event has crashed.
 Returns true if the event is dormant and is ready for a C<start()> or C<restart>
 command. Returns false if the event is already running.
 
+=head2 shared_scalar
+
+Returns a reference to a scalar variable that can be shared between the main
+process and the events. This reference can be used within multiple events, and
+multiple shared scalars can be created by each event.
+
+To read from or assign to the returned scalar, you must dereference it. Eg.
+C<$$shared_scalar = 1;>.
+
+=head2 id
+
+Returns the integer ID of the event.
+
+=head2 info
+
+Returns a hash reference containing various data about the event. Eg.
+
+    $VAR1 = {
+        'shared_scalars' => {
+            '0x55435449' => \'hello, world!,
+            '0x43534644' => \98
+         },
+        'pid' => 6841,
+    };
+
+=head2 events
+
+This is a class method that returns a hash reference that contains the data of
+all existing events. Call it with C<Async::Event::Interval::events()>.
+
+    $VAR1 = {
+        '0' => {
+            'shared_scalars' => {
+                '0x555A4654' => \'hello, world',
+                '0x4C534758' => \98
+             },
+            'pid' => 11859,
+        },
+        '1' => {
+            'pid' => 11860
+        }
+    };
+
 =head1 EXAMPLES
 
 =head2 Run Once
 
 Send in an interval of zero (C<0>) to have your event run a single time. Call
-C<start()> repeatedly for numerous runs.
+C<start()> repeatedly for numerous individual/one-off runs.
 
     use Async::Event::Interval
 
@@ -247,11 +368,33 @@ C<start()> repeatedly for numerous runs.
 
     $event->start if $event->waiting;
 
+=head2 Event Suicidal Timeout
+
+You can have your callback commit suicide if it takes too long to run. We use
+Perl's C<$SIG{ALRM}> and C<alarm()> to do this. In your main application, you
+can check the status of the event and restart it or whatever else you need.
+
+    my $event_timeout = 30;
+
+    my $event = Async::Event::Interval->new(
+        30,
+        sub {
+            local $SIG{ALRM} = sub { print "Committing suicide!\n"; kill 9, $$; };
+
+            alarm $event_timeout;
+
+            # Do stuff here. If it takes 30 seconds, we kill ourselves
+
+            alarm 0;
+        },
+    );
+
 =head2 Event Parameters
 
 You can send in a list of parameters to the event callback. Changing these
 within the main program will have no effect on the values sent into the
-event itself.
+event itself. These parameter variables are copies and are not shared. For
+shared variables, see L</shared_scalar>.
 
     use Async::Event::Interval
 
@@ -268,45 +411,21 @@ event itself.
         print "$one, $two, $three\n";
     }
 
-=head2 Shared Data
-
-A timed event where the event callback shares a scalar variable with the main
-program.
-
-    use Async::Event::Interval;
-    use IPC::Shareable;
-
-    tie my $scalar, 'IPC::Shareable', 'KEY', {create => 1, destroy => 1};
-    $scalar = "hello";
-
-    my $event
-        = Async::Event::Interval->new(10, \&callback);
-
-    sub callback {
-        $scalar = "hello, world!";
-    }
-
 =head2 Event crash: Restart event
 
     use warnings;
     use strict;
-    use feature 'say';
 
     use Async::Event::Interval;
 
-    my $event = Async::Event::Interval->new(
-        2,
-        sub {
-            kill 9, $$;
-        },
-    );
+    my $event = Async::Event::Interval->new(2, sub { kill 9, $$; });
 
     $event->start;
 
-    sleep 1; # do stuff
+    sleep 1; # Do stuff
 
     if ($event->status == -1){
-        say "event crashed, restarting";
+        print "Event crashed, restarting\n";
         $event->restart;
     }
 
@@ -314,25 +433,16 @@ program.
 
     use warnings;
     use strict;
-    use feature 'say';
 
     use Async::Event::Interval;
 
-    my $event = Async::Event::Interval->new(
-        2,
-        sub {
-            kill 9, $$;
-        },
-    );
+    my $event = Async::Event::Interval->new(1.7, sub { kill 9, $$; });
 
     $event->start;
 
-    sleep 1; # do stuff
+    sleep 1; # Do stuff
 
-    if ($event->status == -1){
-        say "event crashed, can't continue...";
-        exit;
-    }
+    die "Event crashed, can't continue" if $event->status == -1;
 
 =head1 AUTHOR
 
